"""
.. module:: __init__

   :synopsis: This is the top-level of the package. Defines version of software.

.. moduleauthor:: Scott W. Fleming <fleming@stsci.edu>
"""

from __future__ import absolute_import

__version__ = '1.37.1'
