"""
.. module:: test_placeholder
   :synopsis: Placeholder unit test until actual tests are implemented.

.. moduleauthor:: Scott W. Fleming
"""

import pytest

def test_placeholder():
    assert 1 == 1
